function _extends() { _extends = Object.assign ? Object.assign.bind() : function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

import React from 'react';
import { createPortal } from 'react-dom';
import PropTypes from 'prop-types';
import { Button } from '@cloudflare/elements';
import { createComponent } from '@cloudflare/style-container';
import ReactTooltip from 'react-tooltip';
import uniqueId from 'lodash.uniqueid';
var DELAY_SHOW_MILLIS = 400;

var getBackgroundColor = (theme, type) => {
  if (type === 'dark') {
    return "".concat(theme.colors.gray[2], " !important");
  }

  if (type === 'success') {
    return "".concat(theme.colors.green[6], " !important");
  }

  return 'inherit';
};

export var tooltipStyles = _ref => {
  var {
    theme,
    type,
    place
  } = _ref;
  return {
    '&:after': {
      borderTopColor: place === 'top' ? getBackgroundColor(theme, type) : 'inherit',
      borderRightColor: place === 'right' ? getBackgroundColor(theme, type) : 'inherit',
      borderLeftColor: place === 'left' ? getBackgroundColor(theme, type) : 'inherit',
      borderBottomColor: place === 'bottom' ? getBackgroundColor(theme, type) : 'inherit'
    },
    color: "".concat(theme.colors.white, " !important"),
    backgroundColor: getBackgroundColor(theme, type),
    opacity: '1 !important',
    fontSize: "".concat(theme.fontSizes[1], "px !important"),
    borderRadius: '2px !important',
    maxWidth: '21rem !important',
    paddingLeft: "".concat(theme.space[3], "px !important"),
    paddingRight: "".concat(theme.space[3], "px !important"),
    paddingTop: "".concat(theme.space[2], "px !important"),
    paddingBottom: "".concat(theme.space[2], "px !important"),
    fontWeight: 'normal !important',
    lineHeight: '1.5 !important',
    pointerEvents: 'auto !important',
    verticalAlign: 'baseline !important',
    whiteSpace: 'normal',
    zIndex: '1300 !important',
    '&:hover': {
      visibility: 'visible !important',
      opacity: '1 !important'
    },
    '& a': {
      color: theme.colors.white,
      textDecoration: 'underline',
      ':hover': {
        textDecoration: 'none'
      }
    }
  };
};

class Tooltip extends React.PureComponent {
  constructor(props) {
    super(props);
    this.id = uniqueId('cf_component_tooltip_');
    this.escFunction = this.escFunction.bind(this);
    this.delayTimer = null;
  }

  escFunction(event) {
    if (event.key === 'Escape') {
      ReactTooltip.hide(this.tooltipElement);
    }
  }

  componentDidUpdate() {
    // full rebuild is required when tooltip is already displayed
    // but we changed type of position of the tooltip
    if (this.props.forceRebuild) {
      ReactTooltip.hide(this.tooltipElement);
      setTimeout(() => ReactTooltip.show(this.tooltipElement), 0);
    }
  }

  componentDidMount() {
    document.addEventListener('keydown', this.escFunction, {
      capture: true
    });
  }

  componentWillUnmount() {
    document.removeEventListener('keydown', this.escFunction, {
      capture: true
    });
    clearTimeout(this.delayTimer);
  }

  render() {
    var {
      disable,
      place,
      message,
      children,
      className,
      delayHide,
      delayShow,
      afterHide,
      afterShow,
      getContent,
      type,
      anchored,
      buttonProps,
      isCapture,
      display,
      ariaLabel,
      Component = Button,
      componentRef,
      portal = false
    } = this.props;
    var id = this.props.id ? this.props.id : this.id;
    return /*#__PURE__*/React.createElement(React.Fragment, null, portal ? /*#__PURE__*/createPortal( /*#__PURE__*/React.createElement(ReactTooltip, {
      id: id,
      disable: disable,
      place: place,
      type: type,
      effect: anchored ? 'solid' : 'float',
      className: className,
      isCapture: isCapture,
      delayHide: delayHide ? delayHide : null,
      afterShow: afterShow,
      afterHide: afterHide,
      getContent: [() => message || getContent()],
      onClick: this.props.onClick,
      event: "click",
      role: "status"
    }), document.body) : /*#__PURE__*/React.createElement(ReactTooltip, {
      id: id,
      disable: disable,
      place: place,
      type: type,
      effect: anchored ? 'solid' : 'float',
      className: className,
      isCapture: isCapture,
      delayHide: delayHide ? delayHide : null,
      afterShow: afterShow,
      afterHide: afterHide,
      getContent: [() => message || getContent()],
      onClick: this.props.onClick,
      event: "click",
      role: "status"
    }), /*#__PURE__*/React.createElement(Component, _extends({
      innerRef: ref => {
        this.tooltipElement = ref;

        if (componentRef !== null && componentRef !== void 0 && componentRef.hasOwnProperty('current')) {
          componentRef.current = ref;
        } else if (typeof componentRef === 'function') {
          componentRef(ref);
        }
      },
      display: display ? display : 'flex',
      pointerEvents: "bounding-box",
      padding: 0,
      backgroundColor: "transparent",
      cursor: "pointer",
      borderWidth: 0,
      "data-tip": true,
      "data-for": id,
      type: "button",
      onFocus: () => ReactTooltip.show(this.tooltipElement),
      onBlur: () => ReactTooltip.hide(this.tooltipElement),
      color: "gray.1",
      onClick: e => {
        e.stopPropagation();
      },
      onMouseOver: () => {
        if (delayShow) {
          this.delayTimer = setTimeout(() => {
            ReactTooltip.show(this.tooltipElement);
          }, DELAY_SHOW_MILLIS);
        } else {
          ReactTooltip.show(this.tooltipElement);
        }
      },
      onMouseOut: () => {
        clearTimeout(this.delayTimer);
        ReactTooltip.hide(this.tooltipElement);
      },
      "aria-controls": id,
      "aria-label": ariaLabel,
      "aria-describedby": id
    }, buttonProps), children));
  }

}

Tooltip.propTypes = {
  id: PropTypes.string,
  message: PropTypes.node.isRequired,
  place: PropTypes.oneOf(['top', 'right', 'bottom', 'left']),
  type: PropTypes.oneOf(['dark', 'success', 'warning', 'error', 'info', 'light']),
  anchored: PropTypes.bool,
  className: PropTypes.string,
  isCapture: PropTypes.bool,
  children: PropTypes.node,
  disable: PropTypes.bool,
  delayHide: PropTypes.number,
  delayShow: PropTypes.bool,
  afterHide: PropTypes.func,
  afterShow: PropTypes.func,
  buttonProps: PropTypes.object,
  getContent: PropTypes.func,
  forceRebuild: PropTypes.bool,
  onClick: PropTypes.func,
  display: PropTypes.string,
  ariaLabel: PropTypes.string,
  Component: PropTypes.func,
  // Allow the Component element to be referenced from a parent. innerRef
  // is swallowed by createComponent.
  componentRef: PropTypes.oneOfType([PropTypes.func, PropTypes.object]),
  portal: PropTypes.bool
};
Tooltip.defaultProps = {
  disable: false,
  place: 'top',
  type: 'dark',
  anchored: true,
  isCapture: true,
  forceRebuild: false,
  delayShow: true,
  portal: false
};
Tooltip.displayName = 'Tooltip';
export default createComponent(tooltipStyles, Tooltip);